import { DateTime as O, Duration as J } from "luxon";
var S = /* @__PURE__ */ ((e) => (e[e.Monday = 1] = "Monday", e[e.Tuesday = 2] = "Tuesday", e[e.Wednesday = 3] = "Wednesday", e[e.Thursday = 4] = "Thursday", e[e.Friday = 5] = "Friday", e[e.Saturday = 6] = "Saturday", e[e.Sunday = 7] = "Sunday", e))(S || {}), c = /* @__PURE__ */ ((e) => (e.Week = "WEEK", e.MonthExact = "MONTH-EXACT", e.MonthExtended = "MONTH-EXTENDED", e.Days = "DAYS", e))(c || {});
class M extends Error {
  /**
   * Creates a new instance of CalendarRangeError.
   *
   * @param code - The {@link ErrorCode} of the error.
   * @param message - The message of the error.
   */
  constructor(t, s) {
    super(s), this.name = this.constructor.name, this.code = t;
  }
}
class l extends M {
  /**
   * Creates a new instance of InvalidParameterError.
   *
   * @param paramName - The name of the invalid parameter.
   * @param paramValue - The value of the invalid parameter.
   * @param expected - The expected type or format of the parameter value.
   * @param description - An optional description of the validation error.
   *
   * @example
   * ```
   * // create an instance of InvalidParameterError with some sample parameters
   *  const error = new InvalidParameterError(
   * 	"foo",
   * 	42,
   * 	"string",
   * 	"This option is required for the operation.",
   * );
   * // log the error to the console
   * console.log(error)
   * //-> Error: The value of the foo is invalid. You passed 42, but string is expected. This option is required for the operation.
   * ```
   */
  constructor(t, s, r, n) {
    const i = `The value of the ${t} is invalid. 
You passed ${JSON.stringify(
      s
    )}, but ${r} is expected.${n ? ` 
${n}` : ""}`;
    super("INVALID_PARAM", i), this.paramName = t, this.paramValue = s, this.expected = r, this.description = n;
  }
}
class T extends M {
  /**
   * Represents an error that occurs when an argument is not provided but required in the function.
   * @param paramName - The name of the missing argument.
   * @param funcName - The name of the function where the error occurs.
   *
   * @example
   * ```
   * throw new MissingArgumentError("dateRange", "DateRange.getNext()")
   * // -> Error: Missing dateRange argument in DateRange.getNext() function.
   * ```
   */
  constructor(t, s) {
    const r = `Missing ${t} argument in ${s} function`;
    super("MISSING_PARAM", r), this.paramName = t, this.funcName = s;
  }
}
class P extends M {
  /**
   * @example
   * ```
   * throw new EmptyDateRangeError("getNext method");
   * // -> Error: The DateRange object you have passed to the getNext method is empty. You need to create a range using one of the get methods first'.
   * ```
   */
  constructor(t) {
    const s = `The DateRange object you have passed to the ${t} is empty. You need to create a range using one of the get methods first'.`;
    super("EMPTY_DATERANGE", s), this.funcName = t;
  }
}
class A extends l {
  /**
   * Represents an error that occurs when the startOffset parameter is invalid.
   * @param paramValue - The value of the startOffset parameter that caused the error.
   */
  constructor(t) {
    const s = "startOffset", r = "a number >= 0";
    super(s, t, r);
  }
}
class F extends l {
  /**
   * Represents an error that occurs when the endOffset parameter is invalid.
   * @param paramValue - The value of the endOffset parameter that caused the error.
   */
  constructor(t) {
    const s = "endOffset", r = "a number >= 0";
    super(s, t, r);
  }
}
class H extends l {
  /**
   * Represents an error that occurs when the refWeekday parameter is invalid.
   * @param paramValue - The value of the refWeekday parameter that caused the error.
   */
  constructor(t) {
    const s = "refWeekday", r = "a number between 1 and 7.";
    super(s, t, r);
  }
}
class X extends l {
  /**
   * Represents an error that occurs when the refDate parameter is invalid.
   * @param paramValue - The value of the refDate parameter that caused the error.
   */
  constructor(t) {
    const s = "refDate", r = "Date object or luxon DateTime object.";
    super(s, t, r);
  }
}
class E extends l {
  /**
   * Represents an error that occurs when the dateRange parameter is invalid.
   * @param paramValue - The value of the dateRange parameter that caused the error.
   *
   * @example
   * ```
   * throw new InvalidDateRangeError(4)
   * // -> Error: The value of the dateRange parameter is invalid. You passed 4, but a valid, non empty DateRange object is expected.
   * ```
   */
  constructor(t) {
    const s = "dateRange parameter", r = "a valid, non empty DateRange object";
    super(s, t, r);
  }
}
function _(e) {
  return !(e === null || typeof e != "object" || e.constructor !== Object);
}
function $(e) {
  return !!(_(e) && Object.values(e).length === 0);
}
function L(e) {
  return typeof e == "number" && Number.isFinite(e);
}
function q(e) {
  return !(!(e instanceof Date) || isNaN(e.getTime()));
}
function I(e) {
  return !!(e instanceof O && e.isValid);
}
function B(e) {
  return !!(Array.isArray(e) && e.length !== 0 && e.every((t) => I(t)));
}
function N(e) {
  return typeof e == "number" && Number.isInteger(e);
}
function V(e) {
  return !!(q(e) || I(e));
}
function Q(e) {
  if (typeof e != "string")
    return !1;
  try {
    return J.fromObject({ [e]: 1 }).isValid;
  } catch {
    return !1;
  }
}
function U(e) {
  return L(e) && Number.isInteger(e) && e >= 1 && e <= 7;
}
function K(e, t) {
  if (e === void 0)
    throw new T("dateRange", "chunkMonthExtended");
  if (!(e instanceof b) || e.rangeType !== c.MonthExtended)
    throw new Error(
      'The dateRange argument must be an instance of the DateRange class, with a generated range of type: "MONTH-EXTENDED"'
    );
  const s = (t == null ? void 0 : t.nullNextPrevDates) ?? !1;
  if (typeof s != "boolean")
    throw new l(
      "nullNextPrevDates",
      s,
      "boolean"
    );
  const r = [...e.toDateTimes()], n = [];
  for (let i = 0; i < r.length; i += 7) {
    const f = r.slice(i, i + 7);
    n.push(f);
  }
  if (s) {
    const i = [...n], { month: f } = e.refDate;
    return n[0].forEach((d, a) => {
      d.month !== f && (i[0][a] = null);
    }), n[n.length - 1].forEach((d, a) => {
      d.month !== f && (i[i.length - 1][a] = null);
    }), i;
  }
  return n;
}
const w = (e) => {
  if (!_(e) || $(e))
    throw new l("provided argument", e, "a non object");
}, D = (e) => {
  if (!N(e))
    throw new A(e);
}, p = (e) => {
  if (!N(e))
    throw new F(e);
}, C = (e) => {
  if (!U(e))
    throw new H(e);
}, v = (e) => {
  if (!V(e))
    throw new X(e);
}, Z = (e) => {
  if (typeof e != "number")
    throw new l("daysCount param", e, "a number");
  if (!Number.isInteger(e) || e < 0)
    throw new l(
      "daysCount param",
      e,
      "a non-negative integer"
    );
}, z = {
  rangeToAdjust: "rangeToAdjust",
  startOffset: "startOffset",
  endOffset: "endOffset",
  timeUnit: "timeUnit"
}, W = Object.values(z);
function G(e) {
  if (!_(e) || $(e))
    throw new l(
      "options parameter",
      e,
      `an object with all of the following properties: ${W.join(
        ", "
      )}`
    );
  if (Object.keys(e).filter(
    (f) => !(f in z)
  ).length > 0)
    throw new l(
      "ExtendDateRange options parameter",
      e,
      `an object with the following properties: ${W.join(
        ", "
      )}`
    );
  const { endOffset: s, rangeToAdjust: r, startOffset: n, timeUnit: i } = e;
  if (!B(r))
    throw new l(
      "rangeToAdjust",
      r,
      "an array of DateTime"
    );
  if (!Q(i))
    throw new l(
      "timeUnit",
      i,
      "a string that is a DurationUnit",
      'DurationUnit is a string that represents a unit of time, such as "years", "months", "days", etc.`'
    );
  D(n), p(s);
}
function x(e) {
  G(e);
  const { rangeToAdjust: t, timeUnit: s, endOffset: r, startOffset: n } = e;
  if (t.length + r < 1)
    throw new Error(
      `Negative endOffset (${r}) exceeds the date range length (${t.length}).`
    );
  if (t.length + n < 1)
    throw new Error(
      `Negative startOffset (${n}) exceeds the date range length (${t.length}).`
    );
  if (n < 0 && r < 0 && n + r + t.length < 1)
    throw new Error(
      `Negative values of startOffset (${n}) and endOffset (${r}) exceeds the date range length (${t.length}).`
    );
  const i = [...t], f = i[0], d = i[i.length - 1];
  if (n < 0)
    for (let a = 0; a > n; a--)
      i.shift();
  else
    for (let a = 1; a > 0 && a <= n; a++) {
      const o = f.minus({ [s]: a });
      i.unshift(o);
    }
  if (r < 0)
    for (let a = 0; a > r; a--)
      i.pop();
  else
    for (let a = 1; a > 0 && a <= r; a++) {
      const o = d.plus({ [s]: a });
      i.push(o);
    }
  return i;
}
const h = {
  rangeType: void 0,
  daysCount: 0,
  endOffset: 0,
  get refDate() {
    return O.now();
  },
  refWeekday: S.Monday,
  startOffset: 0,
  dateTimes: [],
  isNext: !1,
  isPrevious: !1
};
class b {
  /**
   * DateRange is an entry point for creating and storing a range of dates.
   *
   * @remarks
   * The dates can be generated with get methods.
   *
   * The constructor does not accept any parameters.
   */
  constructor() {
    if (arguments.length > 0)
      throw new l(
        "parameter passed to DateRange instance",
        // rome-ignore lint/style/noArguments: check specifically for empty arguments array
        arguments.length === 1 ? arguments[0] : [...arguments],
        "no parameters",
        "Option parameters should be specified within DateRange methods."
      );
  }
  /**
   * The reference date for this instance.
   */
  get refDate() {
    if (this._refDate === void 0)
      throw new Error(
        "You try to access refDate before it has been initialized. Call one of the getMethods to generate the range and set the refDate."
      );
    return this._refDate;
  }
  /**
   * The reference weekday for this instance.
   */
  get refWeekday() {
    if (this._refWeekday === void 0)
      throw new Error(
        "You try to access refWeekday before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._refWeekday;
  }
  /**
   * The array of luxon DateTimes for this instance.
   *
   * @remarks
   * To get JS Dates use `toJSDates()` method.
   */
  get dateTimes() {
    if (this._dateTimes === void 0)
      throw new Error(
        "You try to access dates before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._dateTimes;
  }
  /**
   * The start offset for this instance.
   */
  get startOffset() {
    if (this._startOffset === void 0)
      throw new Error(
        "You try to access startOffset property before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._startOffset;
  }
  /**
   * The end offset for this instance.
   */
  get endOffset() {
    if (this._endOffset === void 0)
      throw new Error(
        "You try to access endOffset property before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._endOffset;
  }
  /**
   * The days count used with `getDays()` method for this instance.
   */
  get daysCount() {
    if (this._daysCount === void 0)
      throw new Error(
        "You try to access daysCount property before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._daysCount;
  }
  /**
   * The type of range generated for this instance.
   *
   * @remarks
   * See {@link RANGE_TYPE| the RANGE_TYPE enum} for more details.
   *
   * If `undefined`, the range has not been created yet.
   *
   */
  get rangeType() {
    if (this._rangeType === void 0)
      throw new Error(
        "You try to access rangeType property before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._rangeType;
  }
  /**
   * A boolean that indicates whether DateRange is created with a `getNext` method.
   */
  get isNext() {
    if (this._isNext === void 0)
      throw new Error(
        "You try to access isNext property before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._isNext;
  }
  /**
   * A boolean that indicates whether DateRange is created with a `getPrevious` method.
   */
  get isPrevious() {
    if (this._isPrevious === void 0)
      throw new Error(
        "You try to access isPrevious property before it has been initialized. Call one of the getMethods to generate the range and set instance members."
      );
    return this._isPrevious;
  }
  /**
   * Sets or updates the instance members of the DateRange object.
   *
   * @param members - An object containing the properties of the DateRange object to be set or updated.
   * @private
   * */
  _setMembers(t) {
    const {
      rangeType: s,
      refDate: r,
      refWeekday: n,
      startOffset: i,
      endOffset: f,
      daysCount: d,
      dateTimes: a,
      isNext: o,
      isPrevious: u
    } = t;
    s !== this._rangeType && (this._rangeType = s), r !== this._refDate && (this._refDate = r instanceof Date ? O.fromJSDate(r) : r), n !== this._refWeekday && (this._refWeekday = n), i !== this._startOffset && (this._startOffset = i), f !== this._endOffset && (this._endOffset = f), d !== this._daysCount && (this._daysCount = d), o !== this._isNext && (this._isNext = o), u !== this._isPrevious && (this._isPrevious = u), this._dateTimes = a;
  }
  /*================================ Utility METHODS ==============================*/
  /**
   * Checks if a given value is a valid reference date.
   *
   * @remarks A reference date can be either a JS `Date` object or a Luxon `DateTime` object.
   *
   * @param refDate - The value to check.
   * @returns True if the value is a valid reference date, false otherwise.
   */
  isValidRefDate(t) {
    return V(t);
  }
  /**
   * Checks if a given value is a valid weekday (integer from 1 to 7).
   *
   * @param weekday - The value to check.
   * @returns True if weekday is an integer from 1 to 7, false otherwise.
   */
  isValidRefWeekday(t) {
    return U(t);
  }
  /**
   * Checks if a given value is a valid offset (non-negative integer).
   *
   * @param offset - The value to check.
   * @returns True if the value is a non-negative integer, false otherwise.
   */
  isValidOffset(t) {
    return N(t);
  }
  /*================================ CONVERTING METHODS ==============================*/
  /**
   * Returns an array of dates generated for the instance as Luxon DateTime objects.
   */
  toDateTimes() {
    return this.dateTimes;
  }
  /**
   * Returns an array of dates generated for the instance as JS Date objects.
   */
  toDates() {
    return this.dateTimes.map((t) => t.toJSDate());
  }
  /*================================ TIME RANGE METHODS ==============================*/
  /**
   * Creates a single week range.
   *
   * @remarks
   * - By default, the method starts the range on Monday before or on the reference date and ends
   * it on Sunday after or on the reference date.
   *
   * - If not specified, the reference date is set to the current time.
   *
   * - Each date is set to the start of the day (midnight).
   *
   * @param options - An optional object to customize the date range.
   * @returns The `DateRange` with generated dates.
   *
   * @example
   *  ```
   * // get current week starting on Monday
   * const week1 = new DateRange().getWeek();
   *
   * // get week based on a refDate and starting on Sunday
   * const week2 = new DateRange().getWeek({
   * 	refDate: new Date("2023-01-10"),
   * 	refWeekday: WEEKDAY.Sunday,
   * });
   * // Generated dates:👇
   * // Sunday, January 8, 2023 at 12:00:00 AM
   * // Monday, January 9, 2023 at 12:00:00 AM
   * // Tuesday, January 10, 2023 at 12:00:00 AM
   * // Wednesday, January 11, 2023 at 12:00:00 AM
   * // Thursday, January 12, 2023 at 12:00:00 AM
   * // Friday, January 13, 2023 at 12:00:00 AM
   * // Saturday, January 14, 2023 at 12:00:00 AM
   * ```
   */
  getWeek(t) {
    if (t !== void 0) {
      w(t);
      const { refDate: u, refWeekday: m, endOffset: y, startOffset: g } = t;
      u !== void 0 && v(u), m !== void 0 && C(m), g !== void 0 && D(g), y !== void 0 && p(y);
    }
    const {
      refDate: s = h.refDate,
      refWeekday: r = h.refWeekday,
      startOffset: n = h.startOffset,
      endOffset: i = h.endOffset
    } = t || {}, f = {
      rangeType: c.Week,
      dateTimes: [],
      refDate: s,
      refWeekday: r,
      endOffset: i,
      startOffset: n,
      isNext: h.isNext,
      isPrevious: h.isPrevious,
      daysCount: h.daysCount
    };
    let d;
    for (s instanceof Date ? d = O.fromJSDate(s).startOf("day") : d = s.startOf("day"); d.weekday !== r; )
      d = d.minus({ days: 1 });
    const a = f.dateTimes;
    let o = d;
    for (; a.length < 7; )
      a.push(o), o = o.plus({ day: 1 });
    if (n || i) {
      const u = x({
        rangeToAdjust: a,
        timeUnit: "days",
        startOffset: n,
        endOffset: i
      });
      return this._setMembers({
        ...f,
        dateTimes: [...u]
      }), this;
    } else
      return this._setMembers(f), this;
  }
  /**
   *
   * Creates a single month range extended to include the full weeks.
   *
   * @remarks
   *
   * - By default, the method starts the range on Monday before or on the first day of the month and ends
   * it on Sunday after or on the last day of the month.
   *
   * - If not specified, the reference date is set to the current time.
   *
   * - Each date is set to the start of the day (midnight).
   *
   * @param options - An optional object to customize the date range.
   * @returns The `DateRange` with generated dates.
   * @example
   *  ```
   * // Get current month extended to full weeks
   * const monthExtended1 = new DateRange().getMonthExtended();
   *
   * // Get month extended to full weeks based on a refDate and starting on Wednesday
   * const monthExtended2 = new DateRange().getMonthExtended({
   * 	refDate: new Date("2023-01-10"),
   * 	refWeekday: WEEKDAY.Wednesday,
   * });
   * // Generated dates:
   * // Wednesday, December 28, 2022 at 12:00:00 AM -> The first date of the range
   * // Thursday, December 29, 2022 at 12:00:00 AM
   * // Friday, December 30, 2022 at 12:00:00 AM
   * // ...
   * // Tuesday, January 31, 2023 at 12:00:00 AM -> The last date of the range
   * ```
   */
  getMonthExtended(t) {
    if (t !== void 0) {
      w(t);
      const { refDate: g, refWeekday: k, endOffset: R, startOffset: j } = t;
      g !== void 0 && v(g), k !== void 0 && C(k), j !== void 0 && D(j), R !== void 0 && p(R);
    }
    const {
      refDate: s = h.refDate,
      refWeekday: r = h.refWeekday,
      startOffset: n = h.startOffset,
      endOffset: i = h.endOffset
    } = t || {}, f = {
      rangeType: c.MonthExtended,
      dateTimes: [],
      refDate: s,
      refWeekday: r,
      endOffset: i,
      startOffset: n,
      isNext: h.isNext,
      isPrevious: h.isPrevious,
      daysCount: h.daysCount
    }, d = r - 1 === 0 ? 7 : r - 1;
    let a, o;
    s instanceof Date ? (a = O.fromJSDate(s).startOf("month"), o = O.fromJSDate(s).endOf("month")) : (a = s.startOf("month"), o = s.endOf("month"));
    const u = f.dateTimes;
    let m = a;
    for (; m.weekday !== r; )
      m = m.minus({ days: 1 });
    let y = m;
    for (; y.valueOf() < o.valueOf(); )
      u.push(y), y = y.plus({ day: 1 });
    for (; u[u.length - 1].weekday !== d; )
      u.push(u[u.length - 1].plus({ days: 1 }));
    if (n || i) {
      const g = x({
        rangeToAdjust: u,
        timeUnit: "days",
        startOffset: n,
        endOffset: i
      });
      return this._setMembers({
        ...f,
        dateTimes: [...g]
      }), this;
    } else
      return this._setMembers(f), this;
  }
  /**
   * Creates a single month range, from the first to the last day of the month.
   *
   * @remarks
   *
   * - By default, the method starts the range on the first day of the month and ends
   * it on the last day of the month.
   *
   * - If not specified, the reference date is set to the current time.
   *
   * - Each date is set to the start of the day (midnight).
   *
   * @param options - An optional object to customize the date range.
   * @returns The `DateRange` with generated dates.
   * @example
   *  ```
   * // Get current month
   * const month1 = new DateRange().getMonthExact();
   *
   * // Get month with specified refDate
   * const month2 = new DateRange().getMonthExact({
   * 	refDate: new Date("2023-01-10"),
   * });
   * // Generated dates:
   * // Sunday, January 1, 2023 at 12:00:00 AM -> The first date of the range
   * // Monday, January 2, 2023 at 12:00:00 AM
   * // Tuesday, January 3, 2023 at 12:00:00 AM
   * // ...
   * // Tuesday, January 31, 2023 at 12:00:00 AM -> The last date of the range
   *
   * // Get month with specified refDate and offsets
   * const month3 = new DateRange().getMonthExact({
   * 	refDate: new Date("2023-01-10"),
   * 	startOffset: 2,
   * 	endOffset: 2,
   * });
   * // Friday, December 30, 2022 at 12:00:00 AM  -> The range starts 2 days before default first day
   * // Saturday, December 31, 2022 at 12:00:00 AM
   * // Sunday, January 1, 2023 at 12:00:00 AM
   * // ...
   * // Tuesday, January 31, 2023 at 12:00:00 AM
   * // Wednesday, February 1, 2023 at 12:00:00 AM
   * // Thursday, February 2, 2023 at 12:00:00 AM -> The range ends 2 days after default last day
   * ```
   */
  getMonthExact(t) {
    if (t !== void 0) {
      w(t);
      const { refDate: u, endOffset: m, startOffset: y } = t;
      u !== void 0 && v(u), y !== void 0 && D(y), m !== void 0 && p(m);
    }
    const {
      refDate: s = h.refDate,
      startOffset: r = h.startOffset,
      endOffset: n = h.endOffset
    } = t || {}, i = {
      rangeType: c.MonthExact,
      dateTimes: [],
      refDate: s,
      refWeekday: h.refWeekday,
      endOffset: n,
      startOffset: r,
      isNext: h.isNext,
      isPrevious: h.isPrevious,
      daysCount: h.daysCount
    };
    let f, d;
    s instanceof Date ? (f = O.fromJSDate(s).startOf("month"), d = O.fromJSDate(s).endOf("month")) : (f = s.startOf("month"), d = s.endOf("month"));
    const a = i.dateTimes;
    let o = f;
    for (; o.valueOf() < d.valueOf(); )
      a.push(o), o = o.plus({ day: 1 });
    if (r || n) {
      const u = x({
        rangeToAdjust: a,
        timeUnit: "days",
        startOffset: r,
        endOffset: n
      });
      return this._setMembers({
        ...i,
        dateTimes: [...u]
      }), this;
    } else
      return this._setMembers(i), this;
  }
  /**
   * Creates a range of custom number of days.
   *
   * @remarks
   * - The reference date is a starting point of the range.
   *
   * - If not specified, the reference date is set to the current day.
   *
   * - The length of the range can be specified with the `daysCount` property in the `options` object.
   * If not specified, the range will be created with a single date.
   *
   * - Each date is set to the start of the day (midnight).
   *
   * @param options - An optional object to customize the date range.
   * @returns The DateRange with generated dates.
   * @example
   * ```
   * // Get a current date
   * const range1 = new DateRange().getDays();
   *
   * // Get a range of 10 days starting on 2023-01-10
   * const range2 = new DateRange().getDays({
   * 	daysCount: 10,
   * 	refDate: new Date("2023-01-10"),
   * });
   * // Generated dates:
   * // January 10, 2023 at 12:00:00 AM
   * // January 11, 2023 at 12:00:00 AM
   * // January 12, 2023 at 12:00:00 AM
   * // January 13, 2023 at 12:00:00 AM
   * // January 14, 2023 at 12:00:00 AM
   * // January 15, 2023 at 12:00:00 AM
   * // January 16, 2023 at 12:00:00 AM
   * // January 17, 2023 at 12:00:00 AM
   * // January 18, 2023 at 12:00:00 AM
   * // January 19, 2023 at 12:00:00 AM
   * ```
   */
  getDays(t) {
    if (t !== void 0) {
      w(t);
      const { refDate: u, endOffset: m, startOffset: y, daysCount: g } = t;
      u !== void 0 && v(u), y !== void 0 && D(y), m !== void 0 && p(m), g !== void 0 && Z(g);
    }
    const {
      refDate: s = h.refDate,
      startOffset: r = h.startOffset,
      endOffset: n = h.endOffset,
      daysCount: i = 1
    } = t || {}, f = {
      rangeType: c.Days,
      dateTimes: [],
      refDate: s,
      refWeekday: h.refWeekday,
      endOffset: n,
      startOffset: r,
      isNext: h.isNext,
      isPrevious: h.isPrevious,
      daysCount: i
    };
    let d;
    s instanceof Date ? d = O.fromJSDate(s).startOf("day") : d = s.startOf("day");
    const a = f.dateTimes;
    let o = d;
    for (; a.length < i; )
      a.push(o), o = o.plus({ day: 1 });
    if (r || n) {
      const u = x({
        rangeToAdjust: a,
        timeUnit: "days",
        startOffset: r,
        endOffset: n
      });
      return this._setMembers({
        ...f,
        dateTimes: [...u]
      }), this;
    } else
      return this._setMembers(f), this;
  }
  /**
   *
   * Generates the next range based on given one.
   *
   * @remarks
   * The method takes a DateRange object and based on its range type,
   * sets the refDate to a date that can generate the new range.
   *
   * It copies the options used to adjust the given range and applies them to the new range.
   *
   * The method shifts the refDate to the next one as follows:
   * - for a range generated with getDays, the refDate is incremented by the daysCount property.
   * - for a range generated with getWeek, the refDate is incremented by 7 days.
   * - for a range generated with getMonthExact and getMonthExtended, the refDate is set to the first day of the next month.
   *
   * In all cases, the refDate is set to the start of the day (midnight).
   *
   * To check whether the DaterRange is generated with a getNext method, access the `isNext` property of a DateRange instance.
   *
   * @param dateRange - A DateRange object with a generated range.
   * @returns The DateRange with generated dates.
   * @throws an error if DateRange is not provided or the range is not initialized.
   *
   * @example
   * ```
   * // example with a "WEEK" range
   * const week = new DateRange().getWeek({ refDate: new Date("2023-01-10") });
   * week.dateTimes; // dates from Mon, 01/09/2023 to Sun, 01/15/2023
   *
   * const weekNext = new DateRange().getNext(week);
   * weekNext.dateTimes; // dates from Mon, 01/16/2023 to Sun, 01/22/2023
   * ```
   */
  getNext(t) {
    if (t === void 0)
      throw new T("dateRange", "DateRange.getNext()");
    if (!(t instanceof b))
      throw new E(t);
    try {
      t.rangeType;
    } catch {
      throw new P("getNext method");
    }
    const {
      refDate: s,
      startOffset: r,
      endOffset: n,
      refWeekday: i,
      rangeType: f,
      daysCount: d
    } = t;
    switch (f) {
      case c.Week: {
        const o = {
          refDate: s.startOf("day").plus({ days: 7 }),
          refWeekday: i,
          startOffset: r,
          endOffset: n
        };
        return this.getWeek(o), this._isNext = !0, this;
      }
      case c.MonthExact: {
        const o = {
          refDate: s.startOf("month").plus({ month: 1 }),
          startOffset: r,
          endOffset: n
        };
        return this.getMonthExact(o), this._isNext = !0, this;
      }
      case c.MonthExtended: {
        const a = s.startOf("month").plus({ month: 1 }), o = {
          refWeekday: i,
          refDate: a,
          startOffset: r,
          endOffset: n
        };
        return this.getMonthExtended(o), this._isNext = !0, this;
      }
      case c.Days: {
        const a = s.startOf("day").plus({ day: d }), o = {
          daysCount: d,
          endOffset: n,
          startOffset: r,
          refDate: a
        };
        return this.getDays(o), this._isNext = !0, this;
      }
      default:
        throw new Error("not implemented");
    }
  }
  /**  Generates the previous range based on given one.
   *
   * @remarks
   *
   * The method takes a DateRange object and based on its range type,
   * sets the refDate to a date that can generate the new range.
   *
   * It copies the options used to adjust the given range and applies them to the new range.
   *
   * The method shifts the refDate to the previous one as follows:
   *   - for a range generated with getDays, the refDate is decremented by the daysCount property.
   *   - for a range generated with getWeek, the refDate is decremented by 7 days.
   *   - for a range generated with getMonthExact and getMonthExtended, the refDate is set to the first day of the previous month.
   *
   * In all cases, the refDate is set to the start of the day (midnight).
   *
   * To check whether the DaterRange is generated with a getPrevious method, access the `isPrevious` property of a DateRange instance.
   *
   * @param dateRange - A DateRange object with a generated range range.
   * @returns The DateRange object with a generated range of the same type and options as the given one but with shifted dates.
   * @throws an error if DateRange is not provided or the range is not initialized.
   *
   * @example
   * ```
   * // example with a "WEEK" range
   * const week = new DateRange().getWeek({ refDate: new Date("2023-01-10") });
   * week.dateTimes; // dates from Mon, 01/09/2023, to Sun, 01/15/2023
   *
   * const weekPrevious = new DateRange().getPrevious(week);
   * weekPrev.dateTimes; // dates from Mon, 01/02/2023 to Sun, 01/08/2023
   * ```
   */
  getPrevious(t) {
    if (t === void 0)
      throw new T("dateRange", "DateRange.getPrevious");
    if (!(t instanceof b))
      throw new E(t);
    try {
      t.rangeType;
    } catch {
      throw new P("getPrevious method");
    }
    const {
      refDate: s,
      startOffset: r,
      endOffset: n,
      refWeekday: i,
      rangeType: f,
      daysCount: d
    } = t;
    switch (f) {
      case c.Week: {
        const o = {
          refDate: s.startOf("day").minus({ days: 7 }),
          refWeekday: i,
          startOffset: r,
          endOffset: n
        };
        return this.getWeek(o), this._isPrevious = !0, this;
      }
      case c.MonthExact: {
        const o = {
          refDate: s.startOf("month").minus({ month: 1 }),
          startOffset: r,
          endOffset: n
        };
        return this.getMonthExact(o), this._isPrevious = !0, this;
      }
      case c.MonthExtended: {
        const a = s.startOf("month").minus({ month: 1 }), o = {
          refWeekday: i,
          refDate: a,
          startOffset: r,
          endOffset: n
        };
        return this.getMonthExtended(o), this._isPrevious = !0, this;
      }
      case c.Days: {
        const a = s.startOf("day").minus({ day: d }), o = {
          daysCount: d,
          endOffset: n,
          startOffset: r,
          refDate: a
        };
        return this.getDays(o), this._isPrevious = !0, this;
      }
      default:
        throw new Error("not implemented");
    }
  }
}
export {
  b as DateRange,
  c as RANGE_TYPE,
  S as WEEKDAY,
  K as chunkMonthExtended
};
